import os
import sys

import django
from django.conf import settings
from django.test.utils import get_runner
from django.core.management import call_command


if __name__ == "__main__":
    DIRNAME = os.path.dirname(__file__)
    settings.configure(
        DEBUG = True,
        DATABASES = {
            'default': {
                'ENGINE': 'django.db.backends.postgresql_psycopg2',
                'NAME': 'postgres',
                'USER': 'postgres',
                'HOST': os.environ.get("DATABASE_URL", 'localhost'),
                'PORT': 5432,
            }
        },
        MIDDLEWARE = [
            'django.middleware.security.SecurityMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.middleware.common.CommonMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
            'django.middleware.clickjacking.XFrameOptionsMiddleware',
        ],
        EMAIL_BACKEND = 'django.core.mail.backends.dummy.EmailBackend',
        ROOT_URLCONF='simple_feedback.urls',
        INSTALLED_APPS = ('django.contrib.auth',
                        'django.contrib.contenttypes',
                        'django.contrib.sessions',
                        'django.contrib.admin',
                        'simple_feedback'),
        TEMPLATES = [
            {
                'BACKEND': 'django.template.backends.django.DjangoTemplates',
                'DIRS': ['templates'],
                'APP_DIRS': True,
                'OPTIONS': {
                    'context_processors': [
                        'django.template.context_processors.debug',
                        'django.template.context_processors.request',
                        'django.contrib.auth.context_processors.auth',
                        'django.contrib.messages.context_processors.messages',
                    ],
                },
            },
        ],
    )

    django.setup()
    call_command('makemigrations', 'simple_feedback')
    TestRunner = get_runner(settings)
    test_runner = TestRunner()
    failures = test_runner.run_tests(['simple_feedback'])
    sys.exit(bool(failures))



from django.contrib import admin
from django.utils.safestring import mark_safe
from .models import Ticket


class TicketAdmin(admin.ModelAdmin):
    list_filter = ['status']
    list_display = ['subject', 'created', 'status', 'get_user', 'assignee']
    readonly_fields = ['user', 'email', 'subject', 'text', 'meta', 'download_meta_btn']
    search_fields = ['subject']
    exclude = ('meta',)

    def get_user(self, obj):
        if obj.user:
            return '{} <{}>'.format(obj.user.get_full_name(), obj.user.email)
        else:
            return obj.email
    get_user.short_description = 'User'

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        field = super(TicketAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)
        if db_field.name == 'assignee':
            field.label_from_instance = lambda u: '{} <{}>'.format(u.get_full_name(), u.email)
        return field

    def has_module_permission(self, request):
        """ Only available to superusers """
        return request.user.is_superuser

    def download_meta_btn(self, obj):
        markup = '<a href="/api/tickets/{}/meta/">Download Meta JSON </a>'.format(obj.id)
        return mark_safe(markup)
    download_meta_btn.short_description = 'Meta'


admin.site.register(Ticket, TicketAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2018-01-15 18:54
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('simple_feedback', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ticket',
            name='meta',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict),
        ),
    ]



from django.contrib.auth.models import User
from django.core import mail
from rest_framework.reverse import reverse
from rest_framework.test import APITestCase, APIClient
import json
import re
from .models import Ticket


class TestTicketAPI(APITestCase):
    PASSWORD = 'secret'

    def setUp(self):
        super().setUp()

        # Create users
        self.user = User.objects.create_user('user@random.com', email='user@random.com', password=self.PASSWORD)
        self.admin = User.objects.create_superuser('admin@business.com', email='admin@business.com',
                                                   password=self.PASSWORD)

        self.authenticated_ticket = {
            'subject': 'Hello',
            'text': 'I have a problem',
            'meta': {
                'page': '/blog/1'
            }
        }

        self.anon_ticket = {
            'email': 'anonym@mail.com',
            'subject': 'Hello',
            'text': 'I have a problem',
            'meta': {
                'page': '/blog/1'
            }
        }
        # Set clients
        self.user_client = APIClient()
        self.user_client.login(username='user@random.com', password=self.PASSWORD)
        self.anon_client = APIClient()
        self.admin_client = APIClient()
        self.admin_client.login(username='admin@business.com', password=self.PASSWORD)

    def test_create_user_ticket(self):
        url = reverse('ticket-create')
        response = self.user_client.post(url, self.authenticated_ticket, format='json')

        self.assertEqual(response.status_code, 201)
        self.assertEqual(response.data['user'], self.user.id)
        self.assertEqual(response.data['text'], 'I have a problem')
        self.assertEqual(response.data['assignee'], None)

    def test_create_user_ticket_text_required(self):
        url = reverse('ticket-create')
        data = {
            'subject': 'Hello',
        }
        response = self.user_client.post(url, data, format='json')

        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.data, {'text': ['This field is required.']})

    def test_create_user_ticket_subject_required(self):
        url = reverse('ticket-create')
        data = {
            'text': 'I have a problem',
        }
        response = self.user_client.post(url, data, format='json')

        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.data, {'subject': ['This field is required.']})

    def test_create_anonym_ticket(self):
        url = reverse('ticket-create')

        response = self.anon_client.post(url, self.anon_ticket, format='json')

        self.assertEqual(response.status_code, 201)
        self.assertEqual(response.data['user'], None)
        self.assertEqual(response.data['text'], 'I have a problem')

    def test_create_user_ticket_notify_superusers(self):
        url = reverse('ticket-create')

        response = self.user_client.post(url, self.authenticated_ticket, format='json')

        self.assertEqual(response.status_code, 201)
        self.assertEqual(mail.outbox[0].subject, 'New ticket has been submitted')
        self.assertIn('I have a problem', mail.outbox[0].message().as_string())

    def test_tickets_meta_retrieve_403_for_anon(self):
        ticket = Ticket.objects.create(**self.anon_ticket)
        url = reverse('ticket-meta-download', args=[ticket.id])
        response = self.anon_client.get(url)
        self.assertEqual(response.status_code, 403)

    def test_tickets_meta_retrieve_403_for_user(self):
        ticket = Ticket.objects.create(**self.anon_ticket)
        url = reverse('ticket-meta-download', args=[ticket.id])
        response = self.user_client.get(url)
        self.assertEqual(response.status_code, 403)

    def test_tickets_meta_retrieve_200_for_admin(self):
        ticket = Ticket.objects.create(**self.anon_ticket)
        url = reverse('ticket-meta-download', args=[ticket.id])
        response = self.admin_client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_tickets_meta_retrieve_return_meta_as_downloadable_json(self):
        ticket = Ticket.objects.create(**self.anon_ticket)
        url = reverse('ticket-meta-download', args=[ticket.id])
        attachment_name = re.sub(r'\s+', '_', self.anon_ticket.get('subject'))
        response = self.admin_client.get(url)
        content_type = response._headers.get('content-type')
        content_disposition = response._headers.get('content-disposition')
        self.assertEqual(json.loads(response.content), self.anon_ticket.get('meta'))
        self.assertEqual(content_type, ('Content-Type', 'application/json'))
        self.assertEqual(content_disposition, ('Content-Disposition', 'attachment; filename={}.json'.format(attachment_name)))






from django.db import models
from django.db.models.signals import post_save
from django.core import mail
from django.contrib.auth.models import User
from django.template import loader
from django.dispatch import receiver
from django.conf import settings

# This has to stay here to use the proper celery instance with the djcelery_email package
try:
    import scheduler.celery  # noqa
except ImportError:
    pass


class Ticket(models.Model):
    STATUS_CHOICES = (
        ('New', 'New'),
        ('In Progress', 'In Progress'),
        ('Done', 'Done'),
    )

    status = models.CharField(
        max_length=255,
        choices=STATUS_CHOICES,
        default=STATUS_CHOICES[0][0],
        blank=True,
        null=True
    )
    user = models.ForeignKey(User, blank=True, null=True, on_delete=models.SET_NULL)  # anonym if empty
    assignee = models.ForeignKey(User, blank=True, null=True, related_name='tickets', on_delete=models.SET_NULL)
    email = models.EmailField(blank=True, null=True)
    subject = models.TextField()
    text = models.TextField()
    created = models.DateTimeField(auto_now=False, auto_now_add=True)
    modified = models.DateTimeField(auto_now=True, auto_now_add=False)
    meta = models.JSONField(default=dict, blank=True)

    def __str__(self):
        return self.subject


@receiver(post_save, sender=Ticket)
def new_ticket_notification(sender, instance, created, **kwargs):
    if created:
        emails_to_notify = []
        if getattr(settings, 'SIMPLE_FEEDBACK_SEND_TO', False):
            if isinstance(settings.SIMPLE_FEEDBACK_SEND_TO, str):
                emails_to_notify = [settings.SIMPLE_FEEDBACK_SEND_TO]
            elif isinstance(settings.SIMPLE_FEEDBACK_SEND_TO, list):
                for email in settings.SIMPLE_FEEDBACK_SEND_TO:
                    if isinstance(email, str):
                        emails_to_notify.append(email)

        if not emails_to_notify:
            emails_to_notify = list(User.objects.filter(is_superuser=True).values_list('email', flat=True))
        html_template = loader.get_template("email/notify_template.html")
        html_message = html_template.render({'ticket': instance})

        if getattr(settings, 'SIMPLE_FEEDBACK_SEND_MAIL_FUNC_OVERRIDE', False):
            getattr(settings, 'SIMPLE_FEEDBACK_SEND_MAIL_FUNC_OVERRIDE')(message=html_message,
                                                                         recipients=emails_to_notify)
        else:
            mail.send_mail(
                subject='New ticket has been submitted',
                message='',
                from_email=settings.DEFAULT_FROM_EMAIL,
                recipient_list=emails_to_notify,
                html_message=html_message,
                fail_silently=True)



import re
import json

from rest_framework.generics import CreateAPIView, RetrieveAPIView
from rest_framework.permissions import IsAdminUser
from django.shortcuts import get_object_or_404
from django.http import HttpResponse

from .serializers import TicketSerializer
from .models import Ticket


class TicketCreateAPIView(CreateAPIView):
    serializer_class = TicketSerializer

    def perform_create(self, serializer):
        if self.request.user.is_authenticated:
            serializer.validated_data['user'] = self.request.user
        super(TicketCreateAPIView, self).perform_create(serializer)


class TicketMetaRetrieveView(RetrieveAPIView):
    serializer_class = TicketSerializer
    permission_classes = (IsAdminUser,)

    def get(self, request, *args, **kwargs):
        ticket = get_object_or_404(Ticket, id=kwargs.get('pk'))
        response = HttpResponse(json.dumps(ticket.meta, indent=4), content_type='application/json')
        response['Content-Disposition'] = 'attachment; filename={}.json'.format(re.sub(r'\s+', '_', ticket.subject))
        return response



# Generated by Django 2.0.6 on 2018-06-12 15:59

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('simple_feedback', '0003_ticket_assignee'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ticket',
            name='assignee',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='tickets', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-12-28 15:13
from __future__ import unicode_literals

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Ticket',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(blank=True, choices=[('New', 'New'), ('In Progress', 'In Progress'), ('Done', 'Done')], default='New', max_length=255, null=True)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
                ('subject', models.TextField()),
                ('text', models.TextField()),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('meta', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.urls import path
from .views import TicketCreateAPIView, TicketMetaRetrieveView

urlpatterns = [
    path(r"tickets/", view=TicketCreateAPIView.as_view(), name="ticket-create"),
    path(r"tickets/<pk>/meta/", view=TicketMetaRetrieveView.as_view(), name="ticket-meta-download"),
]
